##############################################################################
##  
##                                                                Frank Lübeck
##  Interface to ndg_online.linux.exe and similar (SYMMETRICA) to compute 
##  Specht modules for use in GAP
##  

##############################################################################
##  
##  - Read this file into your GAP session and use the following commands.
##  
##  Usage (partitions must be in decreasing order, permutations in cycle
##  notation):
##  
##  SpechtMatrixSYM(part, perm[, typ])  . . . . returns matrix of permutation 
##                                <perm> in Specht module for partition <part>
##                                (<typ> can be "Rutherford" (default), 
##                                "Boerner" or "Specht")
##  [This uses the programs provided in
##      http://www.mathe2.uni-bayreuth.de/axel/ordrep_engl.html
##  ]
##  
##  DualPartition(part) . . . . . . . returns the dual partition of <part>
##  DimensionSpecht(part) . . . . . . returns dimension of Specht module for
##                                        partition <part>
##  SparseDescriptionMatrix(mat) . . . a short description of a sparse matrix
##  MatrixSparseDescription(desc) . .  the reverse of SparseDescriptionMatrix

##  # Examples:  
##  
##  gap> part := [3,1,1];
##  [ 3, 1, 1 ]
##  gap> DualPartition(part);
##  [ 3, 1, 1 ]
##  gap> part := [4,1,1];    
##  [ 4, 1, 1 ]
##  gap> DualPartition(part);
##  [ 3, 1, 1, 1 ]
##  gap> DimensionSpecht(part);
##  10
##  gap> perm := (1,2)(3,4,5);
##  (1,2)(3,4,5)
##  gap> mat1 := SpechtMatrixSYM(part, perm);
##  [ [ 0, 0, 0, -1, -1, -1, 0, 0, 0, 1 ], [ -1, 0, 1, 0, 1, 0, 0, 1, 0, 0 ], 
##    [ 0, 0, 0, 0, -1, 0, 0, 0, 0, 0 ], [ 0, -1, -1, 0, 0, 1, 0, 0, 1, 0 ], 
##    [ 0, 0, 0, 0, 0, -1, 0, 0, 0, 0 ], [ 0, 0, 1, 0, 0, 0, 0, 0, 0, 0 ], 
##    [ 0, 0, 0, 0, 0, 0, -1, -1, -1, -1 ], [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 ], 
##    [ 0, 0, 0, 0, 0, 0, 0, 1, 0, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 ] ]
##  gap> mat2 := SpechtMatrixSYM(part, perm, "Boerner");
##  [ [ 0, 1, 0, 0, 0, 0, -1, 0, 1, 0 ], [ 0, 0, 0, 1, 0, 0, -1, 0, 0, 1 ], 
##    [ 0, 0, 0, 0, 0, 1, 0, 0, -1, 1 ], [ 1, 0, 0, 0, 0, 0, -1, 1, 0, 0 ], 
##    [ 0, 0, -1, 0, 0, 0, 0, 1, -1, 0 ], [ 0, 0, 0, 0, -1, 0, 0, 1, 0, -1 ], 
##    [ 0, 0, 0, 0, 0, 0, -1, 0, 0, 0 ], [ 0, 0, 0, 0, 0, 0, 0, 0, -1, 0 ], 
##    [ 0, 0, 0, 0, 0, 0, 0, 0, 0, -1 ], [ 0, 0, 0, 0, 0, 0, 0, -1, 0, 0 ] ]
##  gap> mat3 := SpechtMatrixSYM(part, perm, "Specht"); 
##  [ [ 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 1, 0, 0, 0, 0, 0, 0 ], 
##    [ 0, 0, 0, 0, 0, 1, 0, 0, 0, 0 ], [ 1, 0, 0, 0, 0, 0, 0, 0, 0, 0 ], 
##    [ 0, 0, -1, 0, 0, 0, 0, 0, 0, 0 ], [ 0, 0, 0, 0, -1, 0, 0, 0, 0, 0 ], 
##    [ -1, -1, 0, -1, 0, 0, -1, 0, 0, 0 ], [ 0, 1, 1, 0, 0, -1, 0, 0, -1, 0 ], 
##    [ 0, 0, 0, 1, 1, 1, 0, 0, 0, -1 ], [ 1, 0, -1, 0, -1, 0, 0, -1, 0, 0 ] ]
##  gap> d3 := SparseDescriptionMatrix(mat3);
##  [ 10, [ 2, 1 ], [ 4, 1 ], [ 6, 1 ], [ 1, 1 ], [ 3, -1 ], [ 5, -1 ], 
##    [ 1, -1, 2, -1, 4, -1, 7, -1 ], [ 2, 1, 3, 1, 6, -1, 9, -1 ], 
##    [ 4, 1, 5, 1, 6, 1, 10, -1 ], [ 1, 1, 3, -1, 5, -1, 8, -1 ] ]
##  gap> mat3 = MatrixSparseDescription(d3);
##  true

# helper 
DualPartition := function(lambda)
  local res, k, j;
  res := [];
  k := Length(lambda);
  for j in [1..lambda[1]] do
    if j <= lambda[k] then
      res[j] := k;
    else
      k := k-1;
      while j > lambda[k] do
        k := k-1;
      od;
      res[j] := k;
    fi;
  od;
  return res;
end;

# for storing and rereading sparse matrices
# <mat> is a matrix
SparseDescriptionMatrix := function(mat)
  local res, zero, zr, z, i;
  res := [Length(mat[1])];
  zero := Zero(mat[1][1]);
  for z in mat do
    zr := [];
    for i in [1..Length(z)] do
      if z[i] <> zero then
        Add(zr, i);
        Add(zr, z[i]);
      fi;
    od;
    Add(res, zr);
  od;
  return res;
end;
# <sp> is result of previous function, does the inverse
MatrixSparseDescription := function(sp)
  local zlen, res, zr, i, j;
  zlen := sp[1];
  res := NullMat(Length(sp)-1, zlen);
  for i in [1..Length(sp)-1] do
    zr := sp[i+1];
    for j in [1,3..Length(zr)-1] do
      res[i][zr[j]] := zr[j+1];
    od;
  od;
  return res;
end;
    

# dimension of Specht module for partition <mu>, uses the hook formula
DimensionSpecht := function(mu)
  local la, d, f, m, mux, lax, i, j;
  la := DualPartition(mu);
  d := 1;
  f := 1;
  m := 1;
  mux := mu-[0..Length(mu)-1];
  lax := la-[1..Length(la)];
  for i in [1..Length(mu)] do
    for j in [1..mu[i]] do
      d := d * (mux[i] + lax[j]);
      f := f * m;
      m := m + 1;
    od;
  od;
  return f / d;
end;

# args:  part, perm[, typ]  
# where typ can be "Rutherford" (default), "Boerner", "Specht"
SpechtMatrixSYM := function(arg)
  local part, perm, extprog, n, s, inp, outs, outp, path, prg, sp, res, i;
  part := arg[1];
  perm := arg[2];
  if Length(arg) > 2 then
    if arg[3] = "Boerner" then
      extprog := "bdg_online.linux.exe.sh";
    elif arg[3] = "Specht" then
      extprog := "specht_dg_online.linux.exe.sh";
    else
      extprog := "ndg_online.linux.exe.sh";
    fi;
  else
    extprog := "ndg_online.linux.exe.sh";
  fi;
  n := Sum(part);
  part := ShallowCopy(part);
  Sort(part);
  # input for SYMMETRICA
  s := "";
  Append(s, String(Length(part)));
  Add(s, '\n');
  for i in part do
    Append(s, String(i));
    Add(s, '\n');
  od;
  Append(s, String(n));
  Add(s, '\n');
  for i in [1..n] do
    Append(s, String(i^perm));
    Add(s, '\n');
  od;

  inp := InputTextString(s);
  outs := "";
  outp := OutputTextString(outs, false);
  path := DirectoriesSystemPrograms();
  # we have wrapped the program in a small script that redirects the error
  # output to /dev/null
  prg := Filename( path, extprog );
  if prg=fail then
    Print("Executable file ",extprog," not found in your PATH!\n");
    return fail;
  fi;
  Process( DirectoryCurrent(), prg, inp, outp, [] );
  CloseStream(inp);
  CloseStream(outp);
  # translate result to GAP object
  RemoveCharacters(outs,Concatenation(WHITESPACE,"]"));
  sp := SplitString(outs,"","[");
  res := List(sp, a-> List(SplitString(a, "", ":"), Int));
  return res;
end;

